#!/usr/bin/php -q
<?php
// vim: set noexpandtab tabstop=2 softtabstop=2 shiftwidth=2:

// Monitor TrackMania/ManiaPlanet server callbacks
// Created: Slig <slig@laposte.net>
// Updated: Xymph <tm@gamers.org>
require_once('GbxRemote.inc.php');
require_once('rpcutils.inc.php');

// command line options parsing
$options = getopt('crpmkfituwh');
$nochdet = false;
$norcdet = false;
$noprdet = false;
$nompdet = false;
$nopinfo = false;
$nockpnt = false;
$nofinsh = false;
$nochatt = false;
$nochatu = false;
$dowrmup = false;
if (isset($options) && $options) {
	if (isset($options['c']))
		$nochdet = true;
	if (isset($options['r']))
		$norcdet = true;
	if (isset($options['p']))
		$noprdet = true;
	if (isset($options['m']))
		$nompdet = true;
	if (isset($options['k']))
		$nockpnt = true;
	if (isset($options['f']))
		$nofinsh = true;
	if (isset($options['i']))
		$nopinfo = true;
	if (isset($options['t']))
		$nochatt = true;
	if (isset($options['u']))
		$nochatu = true;
	if (isset($options['w']))
		$dowrmup = true;
	if (isset($options['h']))
		printUsage();
}

function printUsage() {
	echo "Usage:\t".basename(__FILE__)." [-a#] [-crpmkfitw] [-h]\n";
	echo "\t-c\tDisable SChallengeInfo details in Begin/EndChallenge\n";
	echo "\t-r\tDisable SChallengeInfo details in Begin/EndRace\n";
	echo "\t-p\tDisable SPlayerRanking details in EndChallenge/Map/Race/Match\n";
	echo "\t-m\tDisable SMapInfo details in Begin/EndMap/Match\n";
	echo "\t-k\tSkip PlayerCheckpoint callbacks completely\n";
	echo "\t-f\tSkip PlayerFinish callbacks completely\n";
	echo "\t-i\tSkip PlayerInfoChanged callbacks completely\n";
	echo "\t-t\tSkip PlayerChat callbacks completely\n";
	echo "\t-u\tReplace chat text with '...' in PlayerChat\n";
	echo "\t-w\tEnable GetWarmUp calls after each StatusChanged\n";
	echo "\t-h\tShow this help information and exit\n";
	exit(1);
}

function addCall() {
	global $multicall;
	$args = func_get_args();
	$methodName = array_shift($args);
	$call = array('methodName' => $methodName, 'params' => $args);
	$index = count($multicall);
	$multicall[] = $call;
	return $index;
}

function showIt($key, $val, $indent, $nodetail = false) {
	if (is_array($val)) {
		echo $indent.'*'.$key." :\n";
		if (!$nodetail) {
			foreach ($val as $key2 => $val2) {
				showIt($key2, $val2, '  '.$indent);
			}
		}
	} else {
		echo $indent.$key.' = '.$val."\n";
	}
}

function getTime() {
	return floor(microtime(true)*1000);
}

function echoTime($tstart) {
	echo sprintf('%7.3f', (getTime()-$tstart)/1000).': ';
}

function echoCallback($r, $arg = '') {
	echo $r[0].'('.$arg.")\n";
}

// get & show system info
$client = connectServer();

$multicall = array();
addCall('EnableCallbacks', true);
addCall('GetServerName');
addCall('GetVersion');
addCall('GetSystemInfo');
addCall('GetStatus');
if ($client->query('system.multicall', $multicall)) {
	$res = $client->getCBResponses();
	showIt('CallBacks', $res, '');
	$res = $client->getResponse();
	showIt('Results', $res, '');
	// remember version
	$version = $res[2][0]['Version'];
} else {
	die('Cannot run multicall - Error '.$client->getErrorCode().': '.$client->getErrorMessage()."\n");
}
echo "\n\n";

// start waiting for callbacks
$tstart = getTime();
while (true) {
	$read = array($client->socket);
	$write = null;
	$except = null;
	$nb = stream_select($read, $write, $except, 0, 1000000);
	if ($nb === false) {
		echo "Error, wait 5s then retry...\n";
		usleep(5000000);
		continue;
	} elseif ($nb > 0) {
		$client->readCB(0);
		$res = $client->getCBResponses();
		//showIt('**CallBacks', $res, '');

		// show all received callbacks
		foreach ($res as $r) {
			// check for reset start timestamp
			if (($r[0] == 'TrackMania.BeginChallenge' && strnatcmp($version, '2.11.21') >= 0) ||
			    $r[0] == 'ManiaPlanet.BeginChallenge' || $r[0] == 'ManiaPlanet.BeginMap') {
				$tstart = getTime();
				echo "\n";
			} elseif ($r[0] == 'TrackMania.BeginRace' && strnatcmp($version, '2.11.21') < 0) {
				$tstart = getTime();
				echo "\n";
			}

			// callback-specific handling
			switch ($r[0]) {
				case 'TrackMania.PlayerCheckpoint':
				case 'ManiaPlanet.PlayerCheckpoint':
					if (!$nockpnt) {
						echoTime($tstart);
						echoCallback($r, @implode(', ', $r[1]));
					}
					break;

				case 'TrackMania.PlayerFinish':
				case 'ManiaPlanet.PlayerFinish':
					if (!$nofinsh) {
						echoTime($tstart);
						echoCallback($r, @implode(', ', $r[1]));
					}
					break;

				case 'TrackMania.PlayerInfoChanged':
				case 'ManiaPlanet.PlayerInfoChanged':
					if (!$nopinfo) {
						echoTime($tstart);
						echoCallback($r);
						showIt('SPlayerInfo', $r[1][0], '         ');
					}
					break;

				case 'TrackMania.StatusChanged':
				case 'ManiaPlanet.StatusChanged':
					echoTime($tstart);
					echoCallback($r, @implode(', ', $r[1]));
					if ($dowrmup && strnatcmp($version, '2.11.11') >= 0 &&
					    $client->query('GetWarmUp')) {
						$res = $client->getResponse();
						echoTime($tstart);
						echo '  WarmUp = '.($res ? '1' : '0')."\n";
					}
					break;

				case 'TrackMania.PlayerChat':
				case 'ManiaPlanet.PlayerChat':
					if (!$nochatt) {
						if ($nochatu) $r[1][2] = '...';
						echoTime($tstart);
						echoCallback($r, @implode(', ', $r[1]));
					}
					break;

				case 'TrackMania.PlayerManialinkPageAnswer':
				case 'ManiaPlanet.PlayerManialinkPageAnswer':
					echoTime($tstart);
					echoCallback($r, @implode(', ', $r[1]));
					if (isset($r[1][3]))
						showIt('SEntryVal', $r[1][3], '         ', false);
					break;

				case 'TrackMania.BeginRace':
				case 'ManiaPlanet.BeginRace':
					echoTime($tstart);
					echoCallback($r);
					showIt('SChallengeInfo', $r[1][0], '         ', $norcdet);
					break;

				case 'TrackMania.EndRace':
				case 'ManiaPlanet.EndRace':
					echoTime($tstart);
					echoCallback($r);
					showIt('SChallengeInfo', $r[1][1], '         ', $norcdet);
					showIt('SPlayerRanking', $r[1][0], '         ', $noprdet);
					break;

				case 'TrackMania.BeginChallenge':
				case 'ManiaPlanet.BeginChallenge':
					echoTime($tstart);
					echoCallback($r);
					showIt('SChallengeInfo', $r[1][0], '         ', $nochdet);
					showIt('WarmUp', $r[1][1], '         ');
					showIt('MatchContinuation', $r[1][2], '         ');
					break;

				case 'TrackMania.EndChallenge':
				case 'ManiaPlanet.EndChallenge':
					echoTime($tstart);
					echoCallback($r);
					showIt('SChallengeInfo', $r[1][1], '         ', $nochdet);
					showIt('SPlayerRanking', $r[1][0], '         ', $noprdet);
					showIt('WasWarmUp', $r[1][2], '         ');
					showIt('MatchContinuesOnNextChallenge', $r[1][3], '         ');
					showIt('RestartChallenge', $r[1][4], '         ');
					break;

				case 'ManiaPlanet.BeginMap':
					echoTime($tstart);
					echoCallback($r);
					showIt('SMapInfo', $r[1][0], '         ', $nompdet);
					if (isset($r[1][1])) {
						showIt('WarmUp', $r[1][1], '         ');
						showIt('MatchContinuation', $r[1][2], '         ');
					}
					break;

				case 'ManiaPlanet.EndMap':
					echoTime($tstart);
					echoCallback($r);
					if (!isset($r[1][1])) {
						showIt('SMapInfo', $r[1][0], '         ', $nompdet);
					} else {
						showIt('SMapInfo', $r[1][1], '         ', $nompdet);
						showIt('SPlayerRanking', $r[1][0], '         ', $noprdet);
						showIt('WasWarmUp', $r[1][2], '         ');
						showIt('MatchContinuesOnNextMap', $r[1][3], '         ');
						showIt('RestartMap', $r[1][4], '         ');
					}
					break;

				case 'ManiaPlanet.BeginMatch':
					echoTime($tstart);
					echoCallback($r);
					if (isset($r[1][0]))
						showIt('SMapInfo', $r[1][0], '         ', $nompdet);
					if (isset($r[1][1]))
						print_r($r[1]);
					break;

				case 'ManiaPlanet.EndMatch':
					echoTime($tstart);
					echoCallback($r);
					if (isset($r[1][1])) {
						if (is_array($r[1][1]))
							showIt('SMapInfo', $r[1][1], '         ', $nompdet);
						else
							showIt('WinnerTeam', $r[1][1], '         ');
					}
					showIt('SPlayerRanking', $r[1][0], '         ', $noprdet);
					if (isset($r[1][2]))
						print_r($r[1]);
					break;

				// generic callback handling
				default:
					echoTime($tstart);
					echoCallback($r, (isset($r[1]) ? (is_array($r[1]) ? @implode(', ', $r[1]) : $r[1]) : ''));
			}
		}
	}
}

$client->Terminate();
?>
