<?php
// vim: set noexpandtab tabstop=2 softtabstop=2 shiftwidth=2:

require_once(dirname($argv[0]) . '/rpcconfig.inc.php');

$api_list = array(
	'', // default/no API
	'2011-08-01',
	'2011-10-06',
	'2012-06-19',
	'2013-04-16',
);

// parse optional API option & value
$rpc_api = '';
$options = getopt('a:');
if (isset($options['a'])) {
	if ($options['a'] == '?') {
		fwrite(STDERR, "Available API versions:\n");
		foreach ($api_list as $key => $val)
			if ($val == '')
				fwrite(STDERR, "\t$key: default/none\n");
			else
				fwrite(STDERR, "\t$key: $val\n");
		exit(1);
	}
	elseif (array_key_exists(intval($options['a']), $api_list))
		$rpc_api = $api_list[intval($options['a'])];
	elseif (in_array($options['a'], $api_list))
		$rpc_api = $options['a'];
	else {
		fwrite(STDERR, 'Unknown API version: '.$options['a']."\n");
		exit(1);
	}

	// drop API option & value from argc/argv
	while ($k = array_search('-a'.$options['a'], $argv)) {
		if ($k !== false)
			unset($argv[$k]);
	}
	while ($k = array_search('-a', $argv)) {
		if ($k !== false) {
			unset($argv[$k]);
			unset($argv[$k+1]);
		}
	}
	$argv = array_merge($argv);
	$argc = count($argv);
}

function clientError($client, $error) {
	fwrite(STDERR, $error.' - Error '.$client->getErrorCode().': '.$client->getErrorMessage()."\n");
	exit(1);
}

function connectServer() {
	global $rpc_ip, $rpc_port, $rpc_user, $rpc_pass, $rpc_api;

	$client = new IXR_Client_Gbx();

	if (!$client->InitWithIp($rpc_ip, $rpc_port))
		clientError($client, 'Connection failed');
	if (!$client->query('Authenticate', $rpc_user, $rpc_pass))
		clientError($client, 'Login failed');

	if ($rpc_api != '') {
		if (!$client->query('SetApiVersion', $rpc_api))
			clientError($client, 'Cannot set API');
	}

	return $client;
}

?>
